#####               SCRIPTS FOR PART 2 

## For Part 2, install the following packages:
#	"car", "gmodels", "multcomp",
# "plotrix", "moments", and" pwr" should have been installed for Part 1
# Also, various functions not part of these packages but used in this script
# may be found in "Functions.R" which can be obtained at "my R website":
#  https://udrive.oit.umass.edu/jlmyers/RScripts/.


#####               Script for Session 8
#####             Between-Subjects Design: One Factor

### Tabling the Data
# Read in the data set, "MemoryData.csv". There are four independent 
# groups of 10 scores each; each group was trained with a different
# method of memorization.
	mem.data=read.csv(file.choose(),header=T)
	head(mem.data)
	attach(mem.data)

# We want to construct a table of data and summary stats.
   dim(mem.data)		  # One use of dim -- to get dimensions
   dim(Score) = c(10,4)	  # Also a way to reshape data
   head(Score)
   info(Score)			  #Score is now a 10 x 4 matrix	
   scores=as.data.frame(Score) 
   colnames(scores) = c("Control", "Loci","Image","Rhyme")
   head(scores)   
   summary(scores)         #mean, median, quartiles for each column
   means = colMeans(scores)
   var = sapply(scores, sd)^2	  #variance(scores) gived covariances as well
                           	  #Therefore, we use squared sd
   table.8_1=round(rbind(scores,means,var),3)
   col1 = c(rep(" ",10),"Means","Variances") #to label the last two rows
   Table_8_1=cbind(col1,table.8_1)
   Table_8_1			   

## Exercise 8.1.
# Plot a curve of the four group means (with standard error bars) 
#  as a function of method


### ANOVA -- Equal Group Sizes
## The F Test
# If all we want is the F test, we could do the following for a
#    one-tactor design with equal n
# The within- and between-df are
	dfw = ncol(scores)*(nrow(scores)-1); dfb=ncol(scores)-1
     F = (nrow(scores)*(sd(means)^2))/mean(var)	# = 3.23
# Then 
	stats=round(cbind(F,dfb,dfw,p=pf(F, dfb, dfw, lower.tail = FALSE)),3)
	stats
# Note the use of the pf function
			
## The aov Function
# We can get the same result without fewer steps.
	mem.aov=aov(Score~factor(Method), data=mem.data)
     summary(mem.aov)			#Gives ANOVA table
# Note conversion of Method to a factor; although not necessary with
# categorical variables, it's good practice. If not done, R issues a
# warning, but does the conversion anyway.
 
## The lm Function
# Another approach
    mem.lm=lm(Score~factor(Method), data=mem.data)
    summary(mem.lm)
# The Intercept estimate is the Control group mean. The other 
# estimates are the group means minus the control mean.
# Other comparisons and contrasts are possible. Session 10 provides
# tests of these. 

# The lm function is most often used in regression analysis.
# If we want the usual ANOVA table,
	anova(mem.lm)

## Eta-Squared
# We can calulate a measure of effect size
   ssb=dfb*(nrow(scores)*(sd(means)^2))
   ssw=dfw*mean(var)
   eta_sq=ssb/(ssb+ssw)
   eta_sq

# Let's clean up. 
# First see what's attached
     search()
     detach(mem.data)
# Now look at the workspace contents.
	ls()
# OR
	LS()
# Now clean up
     rm(col1,dfb,dfw,eta_sq,F,means,mem.aov,mem.lm,Score,
        ssb,ssw,stats,table.8_1,Table_8_1,var)

## NOTE: "scores" will be used later and should be saved for now

## NOTE: LS() is preferable to ls() because it states the class of each
# object, so that we don't remove functions, or data frames we may wish
# to use in the future. The LS and info functions (from Kellerman's website
# "http://psych-swiki.colorado.edu:8080/LearnR/uploads/10/Rprofile.3.site")
# are among the functions in my Functions.R file. 

## Exercise 8.2
#  Use R functions to estimate (a) Cohen's f; (b) omega=squared
# for the mem.data set


### ANOVA -- Unequal Group Sizes
# Read in the data-- mean (over seasons)depression score for each subject
# The file is "MaleEduc.csv"
   male_educ=read.csv(file.choose(),header=T)
   head(male_educ)		#Ignore T and W; used for SPSS analysis
   attach(male_educ)
    
## The tapply and aggregate Functions
   group.means_1=tapply(mean_d, schoolyr,mean)
   group.means_1
   info(group.means_1) #info is in the Functions.R script
# OR
   group.means_2=aggregate(mean_d,list(schoolyr), mean)
   group.means_2
    info(group.means_2)
# Note that tapply and aggregate yield the same means but
#   different mode, class, etc.

# These functions can also provide other statistics; e.g.
   tapply(mean_d, schoolyr,length)  #No. of scores in each group

## Exercise 8.3
# Crreate a table in which the first column is a list of statistics 
# (no. of cases, median, mean, std. error, skewness, kurtosis), and
# there are four more columns, one for each group, with the numerical 
# values.

## NOTE: See the script for Session 2 for help with the following 2 exercises.

## Exercise 8.4:
# Construct a Figure with four boxplots, one for each group.

## Exercise 8.5
# Construct a barplot of the group means with standard errors.

## The Analysis 
# If we do not declare schoolyr as a factor, the aov function yields
# an unintended result:
	summary(aov(mean_d~schoolyr))
# Note the one df for schoolyr. This is NOT what we want.
# Because the schoolyr levels have numerical labels, the aov function
# does not compare the four group means but instead tests the slope of
# a function fitting the data.
# to get the usual ANPOVA test of schoolyr on 3 dfm we must declare 
# schoolyr as a factor. One way is
      summary(aov(mean_d~factor(schoolyr)))
# or, we can explicitly make the conversion to a factor:
	schoolyr=as.factor(schoolyr)
	summary(aov(mean_d~schoolyr))
# or create a new, categolrical, factor:
	schoolyr.f=factor(schoolyr)
	summary(aov(mean_d~schoolyr.f))
# This last approach is useeful if we wish to maintain both
# numerical and categorical factor representations of schoolyr.


### Power of the F test
# We want the n needed for power of the F test
# Is the pwr package installed? If not, type the following command:
#  install.packages("pwr")
# For help,see http://cran.r-project.org/web/packages/pwr/index.html
# Then select the manual for list of functions and their descriptions
# Or if you know which function you want, type ?function name
# Assuming it's installed, load the pwr package:
   library("pwr")

#To determine group size (n) needed for power:
   pwr.anova.test(k=4,n=NULL,f=.4,sig.level=.05,power=.9)
# k = number of groups, f = Cohen's f. n is the required groupize.
# The result is 23.15, or 24 in a group.

## Exerise 8.6
# Assume n = 20 per group, Cohen's f=.4,alpha = .05. Using the pwr function,
# what is the power?


### Trimmed F test for a Independent Groups
# A function, f_trim, is in the functions script, Its use 
# Assumes equal n;
# x is a data frame -- check class(x);
# trim is fraction trimmed from each tail;
# a is number of groups;
# k  is number of scores trimmed from each tail;
# Note -- if quantiles are not integer, discard fraction to decide on k
# E.g., n = 13. Then R has 20% quantiles at 3.4, 10.6, k = 3 because
# there are 3 scores below 3.4, 3 above 10.6
# p1,p2 are trim, 1-trim -- usually .2,.8

# Apply the f_trim function to mem.data. 
#  "scores" should be in workspace from earlier in this session
    f_trim(scores,.2,2,probs=c(.2,.8))
# The result is: 
# [1] "  F     df1      df2      sig"
# [1]  9.518  3.000 20.000  0.000


### Tests Based on Ranks
## The Kruskal-Wallis H test 
# Applied to the "scores" data set
   kruskal.test(scores)
# The result: "Kruskal-Wallis chi-squared = 8.6848, df = 3, p-value = 0.03379"

## Rank-transform Test (F_R)
# First run the K-W test. Then F_R=(N-a)*H/((a-1)*(N-1-H))
	F_R = (36*8.6848)/(3*(39-8.6848)); F_R; pf(F_R,3,36,lower.tail=F)
# F_R = 3.4378; p = .027

### Testing the Homogeneity of Variance Assumption
## Brown-Forsythe Test is in the "car" library folder of the R directory
# reported as Levene's test but "help" motes it is B-F modification
# Install the "car" package if not already done. Then,
	library("car")
# Do the mean depression (mean_d) variances in the Seasons study vary 
# significantly as a function of educational level (schoolyr)?
	leveneTest(mean_d,factor(schoolyr)) 	# p = .00

#  Let's ckean up
      detach(male_educ)
      rm(F_R,group.means_1,group.means_2,schoolyr.f, F_R,scores)


### Finding the Right Transformation
# You still should have in your workspace the RoyerRT data frame. 
# If not, again read in the "Royer_rt.csv". Then,
     attach(RoyerRT)
	head(RoyerRT)

# Calculate a transformation for the mean RT (rt.mean)
# The bottom hinges are 
     bh = aggregate(rt_mean, list(grade,gender),quantile,.25)[,3]
# and the upper hinges are 
	uh = aggregate(rt_mean, list(grade,gender),quantile,.75)[,3]
# and the medians are
	mdn=aggregate(rt_mean,list(grade,gender),median)

# The 25% and 75% quantiles may differ slightly from the hinges but
# are close enough for our purpose. The exact hinges may be obtained 
# from  "by(rt_mean, list(grade,gender),fivenum)" --  A messier output

# Now get the H-spread 
   spread=uh-bh
# Now for the slope of the relation between log(spread) and log(median)
# Note that mdn is a data frame with three columns (grade,gender,rt_mean)
   lm(log(spread)~log(mdn[,3]))  #get slope of relation
#	slope is .9541
# To get a more detailed result:
   summary(lm(log(spread)~log(mdn[,3])))
# According to the result, the appropriate transformation is
   t_rt=rt_mean^(1-.9541) 
   head(t_rt)      		#The first 6 transformed reaction times

##Exercise 8.6 
# Plot boxplots and q-q plots for both rt_mean and its transform, t_rt.
# What effects has the transformation had?

##Rxercise 8.7 
# Do an ANOVAs on rt_mean and t_rt as a function of grade, using only the
# female data.

##Exercise 8.8
# Write a function for Welch's F test; see Box 8.3 (p. 193) in RDASA3

#Clean up
	detach(RoyerRT)
	rm(bh,uh,mdn,RoyerRT,spread,t_rt)
     
################################################################

#####               Script for Session 9
#####            Multi-Factor Between-Subjects Designs

# We will work primarily with the Wiley-Voss data in this session

### Plotting the Means
# Read and reshape the "Wiley.csv" data
	W_V.data = read.csv(file.choose(),header=T,sep=",")
	head(W_V.data)
	ivt.data=W_V.data$ivt   #inference verification test scores
	dim(ivt.data)=c(8,8)    #reshape the ivt scores into subjects x conditions
	ivt.data			  

#Get cell means and std. errors
   mm0=colMeans(ivt.data)  #condition means
   mm=matrix(mm0,2,4,byrow=TRUE) #reshape into format x instruction cell means
   se=sapply(ivt.data,sd)/sqrt(8) #standard errors of the 8 condiitions
   ee=matrix(se,2,4,byrow=TRUE)  #reshape standard errors

#Initial plot of means
   barx=barplot(mm, beside=TRUE,ylim=c(0,100),names.arg=c("N","S","E","A"),
   axis.lty=1,xlab="Instructions",ylab="IVT Mean Scores")

# Add legend and error bars
   legend("topleft", c("Text","Web"),fill=gray(c(0,1)))
   UL=mm+ee
   LL=mm-ee
   arrows(barx,mm,barx,UL,angle=90)
   arrows(barx,mm,barx,LL,angle=90)     

   dev.off()		# Close the plot

  rm(mm,ee,barx,mm0,se,UL,LL,ivt.data)    #Doing some cleanup


### A Two-factor ANOVA
# NOTE: "instruct" was numeric in the original Wiley-Voss data set.
#   Therefore, aov will test for linear relation between
#   ivt and instruct. To avoid this,
     W_V.data$instruct=factor(W_V.data$instruct)
# Now for the ANOVA
	ivt.aov=aov(ivt~format*instruct,data=W_V.data)
	summary(ivt.aov)              
# NOTE: A*B implies A + B + A x B, not multiplication
# A:B implies A x B interaction.

# NOTE: it was not necessary to include "W_V.data$" in the model
# statement because of the "data = W_V.data".
### The Model.Table Function
# Another way to get cell means and std. errors
   model.tables(ivt.aov,"means",se=TRUE)
# outputs all means for levels of format and instruc, and their 
# combinations, as well as the std. error of the differences in means.

# NOTE: The se of the differencesbtween the instruct means is 4.391
# which equals sqrt(154.24*(1/16 + 1/16)). 154.24 is the MSerror 
# of the ANOVA.

# Clean up
	rm(ivt.aov)


### The ANOVA for a 2 x 2 x 2 Design
# Read  "three-way.csv", data from a modificaton of the Wiley-Voss design
	AxBxC = read.csv(file.choose(),header=T,sep=",")
	names(AxBxC)
 
## Exercise 9.1
# Plot the group means and standard error bars for the AxBxC data
# (Might have two panels side by side, one for experts, 
# one for novices. See script for Session 2)

## Exercise 9.2 
# Perform the ANOVA on the AxBxC data

##Exercise 9.3 (if you have RDASA3)
# Section 9.6.2 contains the formula for general omega-squared 
# for Format (in an example similar to our 3-factor one). 
# Using the output from Exercise 9.2, and R, to do the calculation,
# find omega-squared for the 3-factor data set.


### More about Models
## Comparing Models
# A way to decide on pooling is to compare residuals from the complete
# model and the reduced model. Using our 2 x 2 x 2 example,
   abc.aov = aov(y~Format*Exper*Task, data=AxBxC)
   summary(abc.aov)
# Note that only Format, Exper, and their interaction seem to contribute 
# much variability. Therefore, look at the reduced model
   abc2.aov = aov(y~Format*Exper, data=AxBxC)
   summary(abc2.aov)
# If the neglected terms contribute substantial variability, the residuals
#  from the reduced model should be much larger than those from the 
#  complete model, To test this:
     anova(abc2.aov,abc.aov)
# The residual sum of squares for the reduced model is only slightly
# larger than for the complete model (F=.398) so abc2.aov seems 
# appropriate; the Task makes very little contribution to the variability.

## Updating Models
# If we only wished to remove one term -- e.g., the 3-factor interaction --
# we might use the update function:
     abc3.aov=update(abc.aov,.~.-Format:Exper:Task)
     summary(abc3.aov)
# We can also add terms to models that have neglected them.
     abc4.aov=update(abc3.aov,.~.+Format:Exper:Task)
     summary(abc4.aov)

## Exercise 9.4
# To decide whether the Format x Exper interaction should be included
# in our model, test abc3.aov against abc2.aov where abc3.aov involves
# Format + Exper (i.e., no interaction) instead of Format * Exper 
# (which insludes the two main effects and their interaction)

# Clean up
	rm(abc.aov,abc2.aov,abc3.aov,abc4.aov,AxBxC)   

## NOTE: Throughout this session, we have considered only designs with
# equal n's in cells (i.e., "balanced designs"). For the analysis of 
# unbalanced designs, see Session 24.

##############################################################

#####                 Script for Session 10
#####         Contrasting Means in Between-Subjects Designs

# NOTE: Testing contrasts in R can be tricky; in several of the following
# exampless, R constructs contrast matrices using dummy, or index, coding.
# We have to understand how the contrasts are coded, and then choose from
# among different functions that can provide the tests and confidence
# intervals.


### Single-Factor Between Groups Designs: Equal n's and Equal Variances
# First get the data and the factors needed (the "MemoryData.csv" file)
# If you have the "mem.data" data frame (created at the beginning of Session 
# 8) saved in your workspace, you may proceed with the following analyses. 
#   If not, read the Memory Data file in again:
      attach(mem.data)
#Convert Method to a factor:
 	Method = factor(Method)

# Now get the means for the Score variable
      mem.m =aggregate(Score,list(Method),FUN=mean,data=mem.data)
      mem.m
# Note that R lists the means in alphabetical order.

## Testing Some Common Contrasts 
## Method 1: Using the contrast functions in the stats package
#  Contrast each treatment mean with a control mean. First, create
# the (dummy coded) contrast matrix:
     contr.treatment(4,base=1)  
# Creates a matrix where  4 is the number of levels in the contrast and 
# the base is the level other levels are contrasted with (e.g., Control)
# although any base could have been chosen. The result is dummy coding,
# with zeros at the base level and ones indicating the means compared
# with the base. 

# To test the contrasts of Inage, Loci, and Rhyme against the Control:
	contrasts(Method) = contr.treatment
 	summary(lm(Score~Method))

## NOTE: To contrast with --e.g., Rhyme --
#   contrasts(Method) = contr.treatment(4,base=4)

# Look  at the Estimate column of the coefficients. The Intercept is the
# base (Control) mean, and the next three coefficients are the differences
# between that mean and the Image, Loci, and Rhyme means, respectively.
# The remaining columns have the Std. Errors and the test results. The 
# significant intercept result indicates that the control mean differs 
# from zero.

## NOTE: The p values have NOT been adjusted for familywise error rate (FWE).
# We will return to this point shortly.

# Other common contrasts can be tested by the preceding approach.
# Contrasting the means with the grand mean, we construct the contrast matrix
# and proceed as before.
     contr.sum(4)  		#all coefficients sum to zero
     contrasts(Method) = contr.sum
     summary(lm(Score~Method))

# Here the intercept is the grand mean(an unweighted average of the group 
# means if n's are not equal). The t tests whether that mean differs from zero. 
# The remaining estimates are the differences between the GM and each of
# the first three group means.

# Helmert contrasts
     contr.helmert(4)
     contrasts(Method) = contr.helmert
     summary(lm(Score~Method))
   
# NOTE: The intercept estimate is again the grand mean. The next estimate 
# is the Control - Image (C - I) means, divided by 2. The third estimate =
# (C + I - 2*L)/6. The last estimate = (C+I+L-3*R)/12. In each case, 
#  the weights sum to 0 and the sum of the squared numerator coefficients
# divided by the denominator(2, 6, or 12) sum to 1.

# If you request the helmert contrast in SPSS, the result is different. 
# SPSS (as is more typical) tests the first mean against the average
# of the remaining three, the second mean against the average of the
# subsequent ones, etc. Method 2 (see next) is general enough to permit this.

    
## Method 2.  A general approach to contrasts
# NOTE: This requires installation of gmodels. 
# Load the functions
   library("gmodels")

## Testing each mean against the Control in the mem.data frame
#  First construct the contrast matrix using effect coding
       w=rbind(c(1,-1,0,0),c(1,0,-1,0),c(1,0,0,-1));  w

# Next we construct a model and test it
    model=aov(Score~Method, data=mem.data)
    summary(model)

# Now to test the contrasts
   fit.contrast(model,"Method",w,conf.int=.95, df=TRUE)

#  The result is the same as with the test following contr.treatment 
#  with the useful addition of confidence interval bounds.
#  We can construct any contrast matrix and test it with the 
#  gmodels package. For example, the usual SPSS helmert contrast is
         w=rbind(c(3,-1,-1,-1),c(0,2,-1,-1),c(0,0,1,-1));  w
# As before, assuming "model" is still in the workspace,
       fit.contrast(model,"Method",w,conf.int=.95, df=TRUE)

# The approach using fit.contrast is quite flexible, allowing any 
# set of a - 1 contrasts, including those tested by contr.sum and
# contr.treatment. 

##Exercise 10.1
# Using the approach just illustrated (i.e., with fit.contrast),
# and the mem.data, construct and test a contrast matrix that compares 
# the mean of each of the first three levels with the grand mean 
# of the mem.data. Begin by noting that the contrast of m1 (Control
# condition) with the grand mean is m1 - (1/4)(m1+m2+m3+m4)= 0,
# which becomes 3*m1 - (m2+m3_m4), implying c(3,-1,-1,-1) for the first
# contrast.

##Exercise 10.2
# We wish to test
   w=rbind(c(0,1,1,-2),c(2,-1,-1,0),c(0,0,1,-1))
# Carry out the tests and using either a calculator or any software
# you have available,and compare with the results of the tests performed by R. 

##Exercise 10.3 
# Assuming 5 groups, construct a matrix of coefficients to test these 
# contrasts: (a) The difference between the combined A and B
# mean and the C mean; (b) The difference between the combined D snf E
# mean and the C mean; (c)The difference between the combined A and B mean 
# and the combined D and E mwean.


### Correcting Error Rates for Family Size
# If the preceding tests (whether with Method 1 or 2) are used to test more 
# than one contrast, the familywise error rate (FWE) should be adjusted. One
# method using the p.adjust function adjusts the p_value from t_test results
# where p_calue is a vector of the individual p_values. The adjustment Methods
# are "holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr",or "none".

# For example, assume n = 2 (the number of p_values adjusted) and we want
# the Bonferroni correction given the following p values:
     p_value  = c(.023, .049)
     p.adjust(p_value,"bonferroni",2)
# This yields .046 and .094, twice the p_values, indicating that only the 
# first contrast is significant assuming FWE = .05.

#Another example is the Holm correction: Assume
     p_value=c(.587,.005,.0165)
# Then
     p.adjust(p_value,"holm",3)
# The result is [1] 0.587 0.015 0.033. If FWE = .05, the last two are 
# significant. Note that the Holm adjustment multiplies the smallest
# p value by n, the next smallest by n-1, etc.

#  Clean up
     detach(mem.data)
	rm(model,mem.m,p_value,w, Method)
 

### Contrasts with Unequal n's and Equal Variances
# Make sure "gmodels" is loaded --library("gmodels")
# We use the depression scores analyzed previously (male_educ data frame)
# If not still in the workspace, read in "MaleEduc.csv"
	head(male_educ)    #To remind us of the variables; ignore T and W
	attach(male_educ)

# We need to convert schoolyr to a categorical factor; changing to
# categorical labels may also be helpful.
     schoolyr.f=factor(schoolyr,labels=c("HS","C","B","GS"))
# "labels" converts the numerical labels (3,5,7,and 8) to more meaningful
# labels

# The cappropritate oefficients are:
   w2=rbind(c(3,-1,-1,-1),c(109,-33,-37,-39)); w2
# The first contrast weights the means equally whereas the second contrast
# weights according to group size.

# Now to test the contrasts.
   model2 = aov(mean_d~schoolyr.f, data=male_educ)
   summary(model2)
   fit.contrast(model2,"schoolyr.f",w2)


### Contrasts with Unequal n and Unequal Variances
# The function welch.t tests the mean diffference assuming unequal 
# variances. It is in the Functions.R script. The general form is
# welch.t(contrast matrix,dependent variable,factor)
# To run the function on the male_educ data set
      Welch.t(w2,mean_d,schoolyr.f)

## NOTE: The p value is a two-tailed probability.

## Exercise 10.4
# See the Royer_rt.csv file. There are 6 grades with different numbers of 
# students. We wish to contrast (1) the mean of the rt_mean column 
# for the 8th grade males against the mean of the rt_mean scores of
# all the other males; (2) The mean of the fifth and sixth grade males 
# against that of the 3rd and 4th grade males. Do this assuming equal
#  variances and assuming that variances are not equal.

# NOTE: Construct the data frame to be analyzed by using the subset
#  function, and select the rt_mean column. Also, let grade.f=factor(grade)

# Clean up
      detach(male_educ)
      rm(model2,w2, schoolyr.f)
 

### Testing Pairwise Comparisons
## p.adjust Methods
# The methods used in the p.adjust function can also be used for pairwise
# comparisons. We illustrate with Royer's multiplication response times.

# If you have not saved the mult_time data frame, read in the 
# "Royer rt_speed.csv" data
#	mult_time= read.csv(file.choose(),header=T, sep=",")
	head(mult_time)
	attach(mult_time)

#  First, convert the quantitative variable (grade) to a categorical factor.
    grade=factor(grade)

# Now for the test; methods are same as seen previously for "p.adjust" 
   pairwise.t.test(M_Speed,grade,"bonferroni")

#The resulting p values are  6 * the two-tailed p, assuming equal variances
#If you want each test based on only the variances for that comparison,
     pairwise.t.test(M_Speed,grade,"bonferroni",pool.sd=FALSE)
# ?pairwise.t.test provides help with a list of alternative methods
# and useful comments about the methods' relative merits.

## The Tukey HSD test
# Often, we want the Tukey HSD test for pairwise comparisons.
# It is not included in the pairwise.t.test function. To perform the HSD
# test, we first perform the ANOVA, then apply the test.

# First do the ANOVA
    speed.aov=aov(M_Speed~grade)
    summary(speed.aov)

#  Now for the Tukey test:
      TukeyHSD(speed.aov, conf.level = 0.95)
# yields p values and 95% confidence limits 

# NOTE: We can change confidence interval -- e.g., to .9,
# Also, order=TRUE arranges factor levels in increasing order
# so that all differences ("diff" in output) are positive:
# See ?TukeyHSD for other parameters.

# NOTE: Description states that the test has been adjusted for "mildly
# unbalanced" (i.e. slightly unequal n) designs; more precisely, when n's 
# are not equal, both SPSS and R calculate the Tukey-Kramer test when
# TukeyHSD is selected. For example, the upper confidence limit for 
# grade 6-5 is .2101+2.619983*.051233 = .3443296 

# To get critical studentized range values; e.g.,
     qtukey(.05,4,86,lower.tail=F)
# yields  3.705215 (rounded to 3.71 in Box 10.3).
# See help for "qtukey" (for critical values) and "ptukey" (for p values) 
# for further info.

     detach(mult_time)


### The Games-Howell Test for Unequal n's and Unequal Variances
# Our example uses depresion scores as a function of educational level.
# The male_educ data frame should still be in your workspace; if not,
#  read the "MaleEduc.csv" file
	head(male_educ)
	attach(male_educ)
  
# The Games-Howell function in my Functions.R script is lengthy
#  but reproduces the SPSS output for the same data.

# Let's test the function with the attached male_educ data
    Games_Howell(x=mean_d,f=schoolyr)

## Exercise 10.5
# Using both the TukeyHSD and Games-Howell functions,
# compare means for the grades in the Royer rt_speed file.
# Discuss the results and which seems best justified.

     detach(male_educ)


### The Dunnett test: Comparisons Against a Control
#  For this, we use the multcomp package. If not already installed,
#  do so now: install.packages("multcomp") and load:
   library("multcomp")
# We again use the memory data of Table 8_1.
   head(mem.data)
   attach(mem.data)
   method =as.factor(Method)

#First, do the ANOVA
    mem.aov=aov(Score~method,data=mem.data)
    summary(mem.aov)
# Now for the Dunnett test and confidence limits
	mem.dunnett=glht(mem.aov, linfct = mcp(method = "Dunnett"),
		data=mem.data)
	summary(mem.dunnett)       #yields the t's and adjusted p values
	confint(mem.dunnett,.95)   #yields the upper and lower Dunnett bounds

# Compare the p values with those obtained at the beginning of Session 10.
# The earlier p values are much smaller because they are error rates per
# comparison whereas the current p values reflect familywise error rates.

# ?glht provides further details. Also, once installed, the "doc" subfolder 
# under multcomp in the R directory has several pdf files. The examples
# file is particularly useful.

#Suppose we wished to compare the "Rhyme" method with each of the other 3.
#  One way to do this is to define a matrix of coefficients:
	K = rbind(c(1,0,0,-1),c(0,1,0,-1),c(0,0,1,-1));K
	mem.d2=glht(mem.aov, linfct = mcp(method =K ))
	summary(mem.d2)
	confint(mem.d2,.95)

# We could also test these contrasts by
	contrasts(method) = contr.treatment(4,base=4)
      summary(lm(mem.data$Score~method))
# The contr.treatment approach yields tje same estimates of effects and s.e.'s
# But the p values do not take the familywise error rate into account. This 
# method requires further steps to adjust the p values and get confidence 
# intervals.

# The matrix approach is quite general and allows tests of more complex
# contrasts among means. For example,to test helmert contrasts,
	K = rbind(c(3,-1,-1,-1),c(0,2,-1,-1),c(0,0,1,-1))
	mem.h=glht(mem.aov, linfct = mcp(method =K ))
	summary(mem.h)
	confint(mem.h,.95)
# The p values have been adjusted upward; the authors state that
# "instead of using the usual Bonferroni and Holm procedures, 
$ we take the exact correlations between the test statistics into account
# by use of the multivariate t-distribution."


### Scheffe's Method for Controlling Post Hoc Error Rates
# Continuing with the mem.data data frame: 
# Suppose, post hoc, we decide the following contrasts look interesting:
     w1 = c(-1,(1/3),(1/3),(1/3))
     w2 = c(0,0,1,-1)
     w = rbind(w1,w2)

# First get the means, variances, and the contrast value
     m=as.vector(tapply(Score,method,mean)); m
     v =  as.vector(tapply(Score,method,var));v
     mse = mean(v); mse      #mean square error
     psi = w%*%m; psi        # matrix multiplication => contrast value

#Get the df and the critical value of S for alpha = .05
      df1=length(levels(method))-1
     df2=dim(mem.data)[1] - length(levels(method))
     F.crit= qf(.05,df1,df2, lower.tail=FALSE)
     S = sqrt(df1*F.crit); S   #Scheffe critical value

# Calculate t and compare to S and -S
     n = dim(mem.data)[1]/(df1+1)  #Assumes equal n
     s_psi= sqrt(mse*rowSums(w^2)/n); s_psi   #Std.Error of psi
     t = psi/s_psi
     stats = cbind(t,S); stats
# Reject H0 if t > S or less than -S; psi1 (w1 row of output) is significant

# The confidence bounds are
   LB = psi - s_psi*S   
   UB = psi + s_psi*S
   CI = cbind(LB,UB); CI   #Column 1 has the lower bounds, column 2 the upper
# Both the t tests and the CI indicate that w1 is significant and w2 is not 

# Clean up
	detach(mem.data)
	rm(CI,df1,df2,F.crit,K,LB,m,mem.aov,mem.d2,mem.dunnett, mem.h,mem.data,
	   mse,n,psi,S,s_psi,speed.aov,stats,t,UB,v,w,w1,w2,
  	   grade, method,mult_time, male_educ) 

##Exercise 10.6
# Write a function to do the Scheffe test

## Exercise 10.7
# Assume the following set of ontrasts on the memory data:
    psi = cbind(c(3,-1,-1,-1),c(0,2,-1,-1),c(0,0,1,-1))
# Test the mull hypotheses using (a) the fit.contrast
# function (from the gmodels package), (b)the glht function
# from the multcomp package, and (c) Scheffe's test.
# Assume FWE = .05 and adjust p values where necessary.


### Pairwise Comparisons in Multifactor Designs
# We wish to test the pairwise differences among the instruct
# marginal means, using Tukey's HSD test to control FWE
# If the W_V.data data frame is not saved in your workspace, 
# W_V.data = read.csv(file.choose(),header=T); the file is "Wiley.csv".
# Then attach the data frame
     attach(W_V.data)

# Now perform the ANOVA, ensuring that instruct is a factor
	WV.aov=aov(ivt~format*factor(instruct))
	summary(WV.aov)
#  Now for the HSD test.
	TukeyHSD(WV.aov,which="factor(instruct)",.95)


### Contrasts in Multifactor Designs
# We will contrast the combined ivt mean of the first three
# instructional levels against that of the fourth level (c1); and
# also test whether the contrast depends on the level of format (c2).
# One way to test these two contrasts is to use the group variable
# in the W_V.data frame. Then we can use any of the methods presented
# earlier in this session. Assuming that each contrast has been
# selected a priori, and that they represent two different families
# (instruct and instruct x format), we will not adjust the p values.

## NOTE: The group levels correspond to Instructions and Format as follows:
# Group = 1 - 4 => instruct = 1 - 4 & format = 1
# Group = 5 - 8 => instruct = 1 - 4 & format = 2
# Therefore, the first contrast is groups 1,2,3,5,6,7 vs. groups 4 and 8
# The second contrast reverses signs for groups 5 - 8 )format = 2)

# The contrasts are
     c1=c(1,1,1,-3,1,1,1,-3)
     c2=c(1,1,1,-3,-1,-1,-1,3)
     w = rbind(c1,c2)

# We need a different ANOVA model for this
    g.aov=aov(ivt~factor(Group))
    summary(g.aov)
# Now we need to load gmodels (It should have been installed
# earlier in the session; use "search()" to see if it is still loaded)
    library("gmodels")
#Finally, we test the two contrasts
    fit.contrast(g.aov,"factor(Group)",w,conf.int=.95, df=TRUE,data=W_V.data)

## Exercise 10.8  
# Use the multcomp package to test the preceding two contrasts. How
#  do the results differ from those using the gmodels package? Why?

## Exercise 10.9 
#(a) The file EX10_16.csv at my R website contains 
# a 3  3 data set. Table and plot the marginal and cell means. 
#(b) Carry out an ANOVA on the data and present the results in a table.
#(c) With FWE = .05, calculate confidence intervals for pairwise 
# comparisons of the A means; state whichif anycomparisons are significant.

# Clean up (Keep W_V.data; we use it in Session 25)
	detach(W_V.data)
	rm(c1,c2,g.aov,w,WV.aov,W_V.data)


################################################################

###                        Script for Session 11/12
###             Trend Analysis in Between-Subjects Designs

### Polynomial Contrasts in One-Factor Designs
##  Read in the "GSR.csv" data file.
    gsr.data=read.csv(file.choose(),header=T)
    attach(gsr.data)
    names(gsr.data)

## Construct a Matrix of Polynomial Weights
   w = round(contr.poly(5,contrasts = TRUE),4); w
# The column headings of w refer to linear, quadratic, etc.
# The weights are different from those typically found in textbooks
# and published tbles (e.g. Appendix Table C6 of RDASA3).
# The weights there have been divided by the square root of the sum 
# of squared weights. For example, the linear weights in Table C6 are
     zi = c(-2, -1, 0, 1, 2)
#  Squaring and summing, the result is 10. Then
     w2 = zi/sqrt(10); w2    # Note that w2 = w[,1].

## Testing the Contrasts
     stimulus=factor(Stimulus)
     contrasts(stimulus)=w
     summary(lm(GSR~stimulus))
# The intercept is the mean for stimulus = 7, The other four estimates
# result from subtracting the intercept from the mean at those levels.

### Plotting the Estimated Population function
# We plot the estimated population means as a sunction of 
# the stimulus values, assuming the function contains only a linear 
# and quadratic componnent (because these were the only signifcant ones).

## Calculating the Regrssion Coefficients
# Frst get the means
    gsr.m = tapply(GSR,stimulus,mean); gsr.m
# Now for the coefficients 
    b0 = mean(gsr.m)
    b1 = sum(w[,1]*gsr.m)/sum(w[,1]^2)
    b2 = sum(w[,2]*gsr.m)/sum(w[,2]^2)
    coefs = c(b0,b1,b2); coefs
# The predicted means are
     mu = b0+b1*w[,1]+b2*w[,2]; mu  

## Plotting the Predicted Means
# We have the predicted treatment population means. Now to plot our estimated
#  population function.
     x = c(7,9,11,13,15)
     plot(mu~x,type="o",xlab="Stimulus Height",ylab="GSR", xaxp=c(7,15,4))
# xaxp denotes the start value, the end value, and the number of intervals

     dev.off()

##Exercise 11.1
# Use Method 2 (requires the gmodels library), described in Session 10
#  to test the preceding trends. 

# Clean up. Important to avoid confusion with the next set of analyses
	detach(gsr.data)		
	rm(b0,b1,b2,coefs,gsr.data,gsr.m,mu, stimulus,w,w2,x,zi)


### Trend analysis in Two-Factor Designs
# Begin by reading the two-factor "GSR2.csv" file
    gsr2.data=read.csv(file.choose(),header=T)
    attach(gsr2.data)
    head(gsr2.data)

    stimulus = factor(Stimulus)
    category = factor(Category)

## The ANOVA
    gsr2.aov = aov(GSR~stimulus*category,data=gsr2.data)
    summary(gsr2.aov)
# The stimulus main effect is significant; category and the interaction
# are marginal.

## Trend Analysis 
# We have the results for Category and Stimulus, and their interaction
# but now need the results for the polynomial terms. In Session 23, we
# present a general approach within a multiple regression framework. 
# Here, however, we present an approach assuming equal n and equal spacing
# between stimulus levels. If the matrix of polynomial coefficients
# is no longer in your workspace,
     w = contr.poly(5)
#  Them we generate polynomial component of the scores 
#  for each subject as follows:
     x1 = rep(w[,1],18)    # The linear component
     x2 = rep(w[,2],18)    # Quadratic
     x3 = rep(w[,3],18)    # Cubic
     x4 = rep(w[,4],18)    # Quartic
    xpoly=round(cbind(x1,x2,x3,x4),5)
    head(xpoly)
#  Now add them to the gsr2.data data frame
     gsrnu = data.frame(cbind(gsr2.data,xpoly))
#  And to simplify the reference
    xx = category
# Now to create the model and get the results
    poly.aov=aov(GSR~x1+x2+x3+x4+xx*x1+xx*x2+xx*x3+xx*x4,data=gsrnu)
    summary(poly.aov)
# The linear and quadratic (x1,x2) results and their interactions
# with xx (ategory)match those from SPSS . Pooling the sums of
# squares for x3 and x4 yields SPSS' Deviation results; similarly
# for the interaction. 

##Exercise 11.2
# In the Wiley_Voss data, Change the Instruct variable to  Hours =
# 1 to 4 (e.g., Narrative => 1, etc.). Perform a trend analysis, including
# interactions of polynomia; comonents of both factors.

##Exercise 11.3
# Fit the GSR2 data, then test whether the polynomial components for the 
# Control condition differ from the average for the two clinical 
# conditions. For example, does the Control slope differ significantly
#  from the average of the other two slopes?

##Exercise 11.4 
# A method used in some memory studies involves requiring subjects
# to respond yes or no within a predetermined interval to a probe
# of memory. One mathematical model predicts that response accuracy
# will be an S-shaped function of time. Assume that there are seven
# groups of six subjects, each of which is tested with a different
# response interval, ranging from 0.2 seconds to 1.4 in intervals of 0.2.
#(a) Which trend component(s) should be significant?
#(b) EX11_5.csv contains the scores. The independent variable is time
# and the dependent variable is d_prime (d '), a measure of accuracy.
# Test for the trend component(s) you listed in answer to part (a). 
# Also test whether any other component(s) are significant.

## Exercise 11.5
# EX11_7.csv contains a data set from a hypothetical drug experiment. 
# The four levels of the factor A represent drug dosages in mg.
# The dependent variable Y is a performance measure.
#(a) Plot the means of the Y data against the levels of A; 
# include standard error bars.
#(b) Perform an ANOVA on the data, with tests of each of the three
#  polynomial components of A.

# Clean up
	detach(gsr2.data)
	rm(category,gsr2.aov,gsr2.data,gsrnu,poly.aov,stimulus,w,x1,x2,x3,
		x4,xx,xpoly)

	
